package de.dassit;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.io.Writer;
import java.lang.reflect.InvocationTargetException;
import java.net.URI;
import java.net.URISyntaxException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import de.dassit.dbwrapper.Entry;

public class RestServlet extends HttpServlet {

	/**
	 * 
	 */
	private static final long serialVersionUID = -1717266963965891000L;

	private static final boolean searchOnPost = true;

	protected static Map<String, Class<?>> classCache = new HashMap<String, Class<?>>();
	protected static Map<Class<?>, String> classToModule = new HashMap<Class<?>, String>();

	protected static Properties configCache = null;
	protected static Boolean configured = false;

	class Attributes {
		List<String> names;
		List<String> integers;
		List<String> booleans;
		List<String> readonlies;

		boolean isInteger(String name) {
			return integers.contains(name);
		}

		boolean isBoolean(String name) {
			return booleans.contains(name);
		}

		boolean isReadonly(String name) {
			return readonlies.contains(name);
		}

		public boolean isHidden(String attrName) {
			// TODO Auto-generated method stub
			return false;
		}

		public boolean isReadOnly(String key) {
			// TODO Auto-generated method stub
			return false;
		}

	}

	@Override
	protected void service(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ServletException {
		if (request.getMethod().equalsIgnoreCase("PATCH")) {
			doPatch(request, response);
		} else {
			super.service(request, response);
		}
	}

	protected void writeErrorResponse(HttpServletResponse response, int status,
			String msg) throws IOException {
		response.setStatus(status);
		response.setContentType("text/plain;charset=UTF-8");
		response.getWriter().write(msg);
		response.getWriter().write('\n');
	}

	protected void badRequest(HttpServletResponse response, String msg)
			throws IOException {
		writeErrorResponse(response, 400, msg);
	}

	protected void notFound(HttpServletResponse response) throws IOException {
		writeErrorResponse(response, 404, "resource not found");
	}

	Map<String, Boolean> createWantedAttributesMap(String csv) {
		if (csv == null)
			return null;

		csv = csv.trim();

		if (csv.isEmpty())
			return null;

		HashMap<String, Boolean> result = new HashMap<String, Boolean>();

		String[] parts = csv.split(",");
		for (String p : parts) {
			p = p.trim();
			if (p.isEmpty())
				continue;
			result.put(p, true);
		}

		return result;
	}

	@Override
	protected void doGet(HttpServletRequest request,
			HttpServletResponse response) throws IOException {
		String[] parts = parsePath(request, response);
		if (parts == null)
			return;

		String base = parts[0];
		String name = parts[1];

		configure();

		System.out.println("GET " + base + " " + name);

		if (name == null) {
			getAll(request, response, base);
			return;
		}

		Entry factory = getFactoryByName(base);
		if (factory == null) {
			System.out.println("getFactoryByName failed");
			notFound(response);
			return;
		}

		Entry e;
		try {
			e = factory.fetch(name);
		} catch (SQLException e1) {
			e1.printStackTrace();
			badRequest(response, "error during fetch");
			return;
		}

		if (e == null) {
			System.out.println("fetch failed");
			notFound(response);
			return;
		}

		String wantedAttrsCsv = request.getParameter("attrs");

		JSONObject result = new JSONObject();

		result = createJSONFromEntry(e,
				createWantedAttributesMap(wantedAttrsCsv));

		if (result == null) {
			badRequest(response, "error creating JSON");
			return;
		}

		response.setContentType("application/json;charset=UTF-8");

		Writer output = response.getWriter();
		output.write(result.toString());
	}

	@Override
	protected void doDelete(HttpServletRequest request,
			HttpServletResponse response) throws IOException {
		try {
			Entry e = getEntryFromPath(request, response);
			e.delete();
			response.setStatus(204);
			/* 204 - No content */
		} catch (SQLException e1) {
			badRequest(response, "invalid path");
			return;
		} catch (ErrorResponseSent r) {
			return;
		}
	}

	@Override
	protected void doPost(HttpServletRequest request,
			HttpServletResponse response) throws IOException {
		String[] parts = parsePath(request, response);
		if (parts == null)
			return;

		String base = parts[0];
		String name = parts[1];

		if (name != null) {
			badRequest(response, "invalid path");
			return;
		}

		configure();

		try {
			Reader r = request.getReader();
			JSONObject postData = new JSONObject(new JSONTokener(r));

			System.out.println("POST " + base + "/" + name + ":" + postData);

			Entry e = getFactoryByName(base);

			if (e == null) {
				notFound(response);
				return;
			}

			updateEntry(e, postData);
			name = e.getKey();

			if (searchOnPost) {
				Entry existing = e.fetch(name);
				if (existing != null) {
					writeErrorResponse(response, 409, "already exists");
					return;
				}
			}

			e.save();

		} catch (JSONException ex) {
			badRequest(response, ex.getMessage());
			return;
		} catch (SQLException e1) {
			badRequest(response, e1.getMessage());
			e1.printStackTrace();
			return;
		}

		response.setStatus(201);
		setLocationHeader(request, response, name);
	}

	protected void setLocationHeader(HttpServletRequest request,
			HttpServletResponse response, String name) {
		try {
			String encodedUrl = new URI(null, null, request.getRequestURI()
					+ "/" + name, null).toASCIIString();
			response.setHeader("Location", encodedUrl);
		} catch (URISyntaxException e) {
			e.printStackTrace();
		}
	}

	@Override
	protected void doPut(HttpServletRequest request,
			HttpServletResponse response) throws IOException {
		String[] parts = parsePath(request, response);
		if (parts == null)
			return;

		String base = parts[0];
		String name = parts[1];

		if (name == null) {
			badRequest(response, "invalid path");
			return;
		}

		configure();

		boolean created = false;

		try {
			JSONObject postData = parseFromRequest(request, response);

			System.out.println("PUT " + base + "/" + name + ":" + postData);

			if (!postData.has("name") || !postData.get("name").equals(name)) {
				badRequest(response, "name field not present or invalid");
				return;
			}

			Entry factory = getFactoryByName(base);
			if (factory == null) {
				notFound(response);
				return;
			}
			Entry e = factory.fetch(name);

			if (e == null) {
				created = true;
				e = factory;
			} else {
				String header = request.getHeader("If-None-Match");

				if (header != null && header.trim().equals("*")) {
					writeErrorResponse(response, 412, "already exists");
					/* 412 - precondition failed */
					return;
				}
			}

			updateEntry(e, postData);
			e.save();

			if (created) {
				response.setStatus(201);
				setLocationHeader(request, response, name);
			} else {
				response.setStatus(204); /* 204 - no content */
			}

		} catch (JSONException ex) {
			badRequest(response, ex.getMessage());
			return;
		} catch (SQLException e1) {
			writeErrorResponse(response, 500, e1.getMessage());
			e1.printStackTrace();
		} catch (IllegalArgumentException ia) {
			badRequest(response, ia.getMessage());
			return;
		} catch (ErrorResponseSent e1) {
			return;
		}
	}

	protected void getAll(HttpServletRequest request,
			HttpServletResponse response, String base) throws IOException {
		Entry factory = getFactoryByName(base);
		if (factory == null) {
			notFound(response);
			return;
		}

		String wantedAttrsCsv = request.getParameter("attrs");
		Map<String, Boolean> wantedAttrsMap = createWantedAttributesMap(wantedAttrsCsv);

		List<? extends Entry> result;
		try {
			result = factory.fetchAll();
		} catch (SQLException e2) {
			e2.printStackTrace();
			badRequest(response, "bad request");
			return;
		}

		response.setContentType("application/json;charset=UTF-8");
		Writer output = response.getWriter();
		output.write("[");

		boolean first = true;

		for (Entry e : result) {
			if (first) {
				first = false;
			} else {
				output.write(",\n");
			}
			JSONObject j = createJSONFromEntry(e, wantedAttrsMap);
			output.write(j.toString());
		}

		output.write("]");
	}

	/**
	 * Parses the path from the request and fetches an entry from the module. If
	 * an error occurs, like an invalid path or the entry was not found, an
	 * error document is sent and a status code is set via the response object.
	 * 
	 * @param request
	 * @param response
	 * @return an Entry object containing data or null if no entry was found
	 * @throws IOException
	 * @throws SQLException
	 */
	protected Entry getEntryFromPath(HttpServletRequest request,
			HttpServletResponse response) throws IOException,
			ErrorResponseSent, SQLException {
		String[] parts = parsePath(request, response);
		if (parts == null) {
			badRequest(response, "invalid path");
			throw new ErrorResponseSent();
		}

		String base = parts[0];
		String name = parts[1];

		if (name == null) {
			badRequest(response, "invalid path");
			throw new ErrorResponseSent();
		}

		configure();

		Entry factory = getFactoryByName(base);
		if (factory == null) {
			notFound(response);
			throw new ErrorResponseSent();
		}

		Entry result = factory.fetch(name);
		if (result == null) {
			notFound(response);
			throw new ErrorResponseSent();
		}
		return result;
	}

	protected void doPatch(HttpServletRequest request,
			HttpServletResponse response) throws IOException {
		// writeErrorResponse(response, 501, "not implemented");

		// a patch document looks like this:
		// { "rename": "newName" }
		// or
		// { "add": { "key1": [ "value1", "value2" ], "key2": [ "value1" ]}}
		// or
		// { "delete": { "key1": [ "value1", "value2" ], "key2": [ "value1" ]}}
		// or
		// { "replace": { "key1": [ "value1", "value2" ], "key2": [ "value1" ]}}

		try {
			JSONObject patch = parseFromRequest(request, response);
			Entry e = getEntryFromPath(request, response);

			System.out.println("PATCH " + request.getPathInfo() + ":" + patch);

			if (patch.has("replace")) {
				JSONObject replace = (JSONObject) patch.get("replace");
				updateEntry(e, replace);
				e.save();
			} else if (patch.has("rename")) {
				String newName = (String) patch.get("rename");
				e.rename(newName);
				setLocationHeader(request, response, newName);
			} else {
				badRequest(response, "malformed patch document");
				return;
			}
		} catch (ClassCastException c) {
			badRequest(response, "malformed patch document");
			return;
		} catch (JSONException j) {
			badRequest(response, "malformed patch document");
			return;
		} catch (SQLException n) {
			badRequest(response, n.getMessage());
			return;
		} catch (IllegalArgumentException a) {
			badRequest(response, a.getMessage());
			return;
		} catch (ErrorResponseSent r) {
			return;
		}

		response.setStatus(204); /* 204 - No content */
	}

	interface Perform {
		void perform(Entry item, String key, String value);
	}

	protected JSONObject parseFromRequest(HttpServletRequest request,
			HttpServletResponse response) throws IOException, ErrorResponseSent {
		JSONObject postData;
		try {
			postData = new JSONObject(new JSONTokener(request.getReader()));
		} catch (JSONException j) {
			writeErrorResponse(response, 400, j.getMessage());
			throw new ErrorResponseSent();
		}
		return postData;
	}

	protected void configure() {
		if (configured) {
			return;
		}

		synchronized (configured) {
			// Properties connectionConfig = getConfig("connection");
			configured = true;
		}
	}

	protected String[] parsePath(HttpServletRequest request,
			HttpServletResponse response) throws IOException {
		request.setCharacterEncoding("UTF-8");
		String path = request.getPathInfo();

		if (path == null) {
			badRequest(response, "invalid path");
			return null;
		}

		String[] parts = path.split("/");
		if (!(parts.length == 2 || parts.length == 3)) {
			badRequest(response, "invalid path");
			return null;
		}

		String[] result = new String[2];
		result[0] = parts[1];
		if (parts.length == 3)
			result[1] = parts[2];
		else
			result[1] = null;
		return result;
	}

	protected JSONObject createJSONFromEntry(Entry e,
			Map<String, Boolean> wantedAttributes) {

		JSONObject result = new JSONObject();

		Attributes attrs = getAttributeNames(e);

		String keyName = e.getKeyName();
		if (wantedAttributes != null && !wantedAttributes.containsKey(keyName)) {
			wantedAttributes.put(keyName, true);
		}

		for (String attrName : attrs.names) {
			if (attrs.isHidden(attrName))
				continue;

			if (wantedAttributes != null
					&& !wantedAttributes.containsKey(attrName))
				continue;

			String val = null;
			Object o = null;
			try {
				val = BeanUtils.getProperty(e, attrName);
			} catch (IllegalAccessException e1) {
				e1.printStackTrace();
			} catch (InvocationTargetException e1) {
				e1.printStackTrace();
			} catch (NoSuchMethodException e1) {
				e1.printStackTrace();
			}

			if (val != null) {
				if (attrs.isBoolean(attrName)) {
					o = new Boolean(val);
				} else if (attrs.isInteger(attrName)) {
					o = new Integer(val);
				} else {
					o = val;
				}
			}

			if (o != null) {
				result.put(attrName, o);
			}

		}
		return result;
	}

	private Attributes getAttributeNames(Entry ent) {
		String module = classToModule.get(ent.getClass());

		Properties conf = getModulesConfig();
		String p = conf.getProperty(module + ".attrs");
		String[] attrs = p.split(",");

		List<String> names = new ArrayList<String>();
		List<String> booleans = new ArrayList<String>();
		List<String> integers = new ArrayList<String>();
		List<String> readonlies = new ArrayList<String>();

		for (String a : attrs) {
			String finalAttrName = a;

			int typeSpecIndex = a.indexOf(':');
			if (typeSpecIndex != -1) {
				finalAttrName = a.substring(0, typeSpecIndex);
				String typeSpec = a.substring(typeSpecIndex + 1);

				if (typeSpec.startsWith("i")) {
					integers.add(finalAttrName);
				}
				if (typeSpec.startsWith("b")) {
					booleans.add(finalAttrName);
				}
				if (typeSpec.endsWith("r")) {
					readonlies.add(finalAttrName);
				}

			}
			names.add(finalAttrName);
		}

		Attributes result = new Attributes();
		result.names = names;
		result.integers = integers;
		result.booleans = booleans;
		return result;
	}

	public void updateEntry(Entry e, JSONObject data)
			throws IllegalArgumentException {
		Attributes attrs = getAttributeNames(e);

		Iterator<String> i = data.keys();
		while (i.hasNext()) {
			String key = i.next();
			Object o = data.get(key);

			if (key.equals("name") && e.getKey() != null) {
				continue;
			}

			if (attrs.isReadOnly(key))
				continue;

			if (attrs.isBoolean(key)) {
				if (!(o instanceof Boolean)) {
					throw new IllegalArgumentException(key + " must be boolean");
				}
			} else if (attrs.isInteger(key)) {
				if (!(o instanceof Integer || o instanceof Long)) {
					throw new IllegalArgumentException(key
							+ " must be an integer");
				}
			}

			if (o instanceof JSONArray || o instanceof JSONObject) {
				throw new IllegalArgumentException(key + " field invalid");
			}

			System.out.println(e.getKey() + "(" + e.getClass().getName()
					+ "): " + key + "=" + o);

			if (PropertyUtils.isWriteable(e, key)) {
				try {
					BeanUtils.setProperty(e, key, o);
				} catch (IllegalAccessException e1) {
					e1.printStackTrace();
					throw new IllegalArgumentException("error accessing " + key
							+ ": " + e1.getMessage());
				} catch (InvocationTargetException e1) {
					e1.printStackTrace();
					throw new IllegalArgumentException("error accessing " + key
							+ ": " + e1.getMessage());
				}
			} else {
				throw new IllegalArgumentException("error accessing " + key);
			}
		}
	}

	public void writeRequestBodyTo(HttpServletRequest req, String path)
			throws IOException {
		File p = new File(path);
		FileWriter fileOut = new FileWriter(p);
		fileOut.write(req.getPathInfo());
		fileOut.write('\n');

		try {
			BufferedReader in = req.getReader();

			String line;

			while ((line = in.readLine()) != null) {
				fileOut.write(line);
				fileOut.write('\n');
			}
		} finally {
			fileOut.close();
		}
	}

	private void connect() {
	}

	Properties getConfig(String name) {
		Properties conf = new Properties();
		try {
			InputStream in = getServletContext().getResourceAsStream(
					"WEB-INF/" + name + ".conf");
			conf.load(in);
			in.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		return conf;
	}

	Entry getFactoryByName(String base) {
		Entry instance = null;

		if (classCache.containsKey(base)) {
			Class<?> c = classCache.get(base);

			try {
				instance = (Entry) c.newInstance();
				configureFactory(base, instance);
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			}
		} else {
			Class<?> c = null;
			Properties modulesConfig = getModulesConfig();

			String className = modulesConfig.getProperty(base);
			if (className == null) {
				log("module configuration not found:" + base);
				return null;
			}
			try {
				c = Class.forName(className);
				instance = (Entry) c.newInstance();

				configureFactory(base, instance);

				synchronized (classCache) {
					classCache.put(base, c);
					classToModule.put(c, base);
				}
				System.out.println("Loaded " + base + " " + className);
			} catch (ClassNotFoundException e) {
				log("module " + base + " class " + className + " not found");
				return null;
			} catch (InstantiationException e) {
				e.printStackTrace();
				return null;
			} catch (IllegalAccessException e) {
				e.printStackTrace();
				return null;
			}
		}

		connect();

		return instance;
	}

	protected Properties getModulesConfig() {
		if (configCache == null) {
			synchronized (this) {
				if (configCache == null) {
					configCache = getConfig("rest-modules");
				}
			}
		}

		return configCache;
	}

	protected void configureFactory(String base, Entry n) {
		String className = null;
		String[] initAttrs = null;

		Properties modulesConfig = getModulesConfig();

		className = modulesConfig.getProperty(base);
		if (className == null) {
			log("module configuration not found:" + base);
			return;
		}

		String init = modulesConfig.getProperty(base + ".initAttrs");
		if (init != null) {
			initAttrs = init.split(",");
		}

		if (initAttrs != null) {
			for (String i : initAttrs) {
				String attrName = i.trim();
				String val = modulesConfig.getProperty(base + "." + attrName);

				log(base + ": setting " + attrName + " to " + val);

				if (val != null) {
					try {
						BeanUtils.setProperty(n, attrName, val);
					} catch (InvocationTargetException e) {
						e.printStackTrace();
					} catch (IllegalAccessException e) {
						e.printStackTrace();
					}
				}
			}
		}
	}

	public class ErrorResponseSent extends RuntimeException {
		private static final long serialVersionUID = 1L;

	}
}